@import url("reset-fonts-grids.css");

html { background: #f9f9f9; color: black; }
body { margin: 10px; }

fieldset { border: 1px solid #ccc; padding: 1em; }
legend { font-weight: bold; margin-left: 1em; padding: 4px; }
textarea, input[type='text'], input[type='password'], select { border: 1px solid #ccc; background: #fff; }
textarea:hover, input[type='text']:hover, input[type='password']:hover, select:hover { border-color: #aaa; }
textarea:focus, input[type='text']:focus, input[type='password']:focus, select:focus { border-color: #888; outline: 2px solid #ffffaa; }
input, select { cursor: pointer; }
input[type='text'] { cursor: text; }
textarea,  input { padding: .3em .4em .15em .4em; }
select { padding: .1em .2em 0 .2em; }
option { padding: 0 .4em; }
table, tr, td, th { border: none; }
hr { height: 1px; color: #aaa; background-color: #aaa; border: 0; margin: .2em 0 .2em 0; }

h1, h2, h3, h4, h5, h6 { font-weight: normal; border-bottom: 1px solid black; }

h1.pageTitle { font-size: 197%; margin: 0.2em 0 .5em; }

h1 { font-size: 153.9%; margin: 1.07em 0 .535em; }
h2 { font-size: 138.5%; margin: 1.14em 0 .57em; }
h3 { font-size: 123.1%; margin: 1.23em 0 .615em; }
h4 { font-size: 116%; margin: 1.33em 0 .67em; }
h5 { font-size: 108%; margin: 1.6em 0 .8em; }
h6 { font-size: 100%; margin: 1.6em 0 .8em; }

strong { font-weight: bold; }
ul { list-style-type: square; }
dt { font-weight: bold; margin-bottom: .1em; }

optgroup{ font-weight: normal; }
abbr, acronym { border-bottom:1px dotted #000; cursor:help; }
em { font-style: italic; }
del { text-decoration: line-through; }

blockquote, ul, ol, dl { margin: 1em; }
ol, ul, dl { margin-left: 2em; }
dl dd { margin-left: 1em; }
th, td { padding: .5em; }
th { font-weight: bold; }
caption { margin-bottom: .5em; text-align: center; }
sup { vertical-align: super; }
sub { vertical-align: sub; }
sub, sup { line-height: 0.3em; }
p, fieldset, table, pre { margin-bottom: 1em; }
button, input[type="checkbox"], input[type="radio"], input[type="reset"], input[type="submit"], input[type="button"] { padding: 1px; }

blockquote { padding: 0 1.6em; color: #666; }

a:link { text-decoration: underline; color: #36c; }
a:visited { text-decoration: underline; color: #99c; }
a:hover { text-decoration: underline; color: #c33; }
a:active,  a:focus { text-decoration: underline; color: #000; }

input.search_term { width: 95% }

button:hover, a.button:hover { background-color:#dff4ff; border:1px solid #c2e1ef; color:#336699; }
a.button:active, button:active { background-color:#6299c5; border:1px solid #6299c5; color:#fff; }

h1 > a:link, h1 > a:active, h1 > a:hover, h1 > a:focus, h1 > a:visited,
h2 > a:link, h2 > a:active, h2 > a:hover, h2 > a:focus, h2 > a:visited,
h3 > a:link, h3 > a:active, h3 > a:hover, h3 > a:focus, h3 > a:visited,
h4 > a:link, h4 > a:active, h4 > a:hover, h4 > a:focus, h4 > a:visited,
h5 > a:link, h5 > a:active, h5 > a:hover, h5 > a:focus, h5 > a:visited,
h6 > a:link, h6 > a:active, h6 > a:hover, h6 > a:focus, h6 > a:visited {
        color: black; text-decoration: none; }

#content { border: 1px solid #ccc; background-color: #fff; padding: 1em; font-size: 108%; }
#content p, #content pre, #content li { line-height: 140%; }

#userbox  { text-align: right; font-weight: bold; margin: 1em; }

#logo { min-height: 50px; }

#sidebar fieldset { background-color: white; margin-bottom: 1em; padding: 0; font-size: 93%; }
#sidebar fieldset, #sidebar fieldset legend { font-weight: normal; }
#sidebar ul { padding: 0; margin: 0; margin-left: 1.6em; line-height: 1.5em; }
#sidebar ul li { color: #888; list-style: square; }

div#TOC { background-color: #f9f9f9; border: 10px solid white; margin: 0.8em; margin-right: 0; padding: 0.4em; }
#TOC ul { padding: 0 0 0 1em; margin: 0; list-style: none; }

#sidebar input, #sidebar select { font-size:  93%; padding: 0.1em; }
#sidebar input[type='submit'] { border: none; background-color: #ccc; color: white; }

#exportbox select { width: 8.5em; border: 1px solid #ccc; padding: 0; }
#exportbox { margin: 0.3em 0 0.5em 0.4em; padding: 0; }

#footer { padding: 1em; color: #888; text-align: center; font-size: 93%; }

#searchform { padding: 0; margin: 0.3em 0 0.5em 0.4em; }
#searchform input[type='text'] { width: 8.5em; border: 1px solid #ccc; }

div#categoryList { padding: 0em; margin: 1em 0 0 0; border: 1px dashed #ccc; }
#categoryList ul > li { display: inline; padding-right: 1em; }

#editform textarea { height: 25em; width: 98%; font-family: monospace; font-size: 93%; }
#editform #logMsg { width: 98%; margin-right: 1em; margin-bottom: 0.3em; }

#goform { padding: 0; margin: 0.3em 0 0.5em 0.4em; }
#goform input[type='text'] { width: 8.5em; border: 1px solid #ccc; }

.search_result { margin-bottom: 15px; }
.search_result .match { margin-bottom: 15px; }

code { font-size: 93%; }
pre.matches { margin: 0; padding: 0; }
#pattern { background-color: yellow; font-weight: bold; }
pre.matches span.highlighted { background-color: yellow; }

.added { background-color: yellow; }
.deleted { text-decoration: line-through; color: gray; }

/* .req is used to hide a honeypot in a form */
.req { display: none; }

ul.messages > li { color: red; list-style: square; font-weight: bold; }

ul.tabs { padding: 0; margin: 0 0 1px 0; }
ul.tabs li { display: inline; border: 1px solid #ccc; border-bottom: none; padding: 0 0.6em 0 0.6em;
    margin: 0 0 0 1.2em; background: white; }
ul.tabs li.selected { border-bottom: 3px solid white; }
ul.tabs li a { text-decoration: none; font-size: 93%; font-weight: bold; margin: 0; color: #36c; }

.index ul { list-style: none; margin: 0; padding: 0; }
.index li { list-style: none; background-position: 0 1px; background-repeat: no-repeat; padding-left: 20px; }
.index li.page { background-image: url(../img/icons/page.png); }
.index li.folder { background-image: url(../img/icons/folder.png); }
.index a { color: #000000; cursor: pointer; text-decoration: none; }
.index a:hover { text-decoration: underline; }

a.updir { font-weight: bold; }

h2.revision { font-size: 100%; color: #888; font-style: italic; border: none; margin: 0 0 0.5em 0; padding: 0; }

div.markupHelp pre { font-size: 77%; overflow: auto; }

.login { display: none; }

/* Style the footnotes; from gwern.net */
#footnotediv div {
background-color: white;
padding: 3px;
padding: 12px;
max-width: 800px;
border: 1px solid #CDBBB5;
box-shadow: #555 0 0 10px;
-webkit-box-shadow: #555 0 0 10px;
-moz-box-shadow: 0 0 10px #555;
}
/* Deal with multiple footnotes one after another; Charuru */
sup + sup { margin-left: 2px; }
